using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrLinearRing : OgrLineString
	{
		public OgrLinearRing(IntPtr ptrLinearRing)
			: base(ptrLinearRing, OGRwkbGeometryType.wkbLinearRing)
		{
		}

		public OgrLinearRing(IntPtr ptrLinearRing, OGRwkbGeometryType geometryType)
			: base(ptrLinearRing, geometryType)
		{
		}

		public OgrLinearRing(OGRwkbGeometryType geometryType)
			: base(geometryType)
		{
		}

		public OgrLinearRing()
			: base(OGRwkbGeometryType.wkbLinearRing)
		{
		}

		/// <summary>
		/// Force linear ring to be closed.
		/// Simply adds the first point as the end point to close the ring.
		/// </summary>
		public void CloseRings()
		{
			int numberPoints = OgrInterop.OGR_G_GetPointCount(this.geometryHandle);
			if (numberPoints < 2)
			{
				return;
			}

			double x0, y0, z0;
			double xend, yend, zend;

			OgrInterop.OGR_G_GetPoint(this.geometryHandle, 0, out x0, out y0, out z0);
			OgrInterop.OGR_G_GetPoint(this.geometryHandle, numberPoints - 1, out xend, out yend, out zend);

			if (x0 != xend || y0 != yend || z0 != zend)
			{
				OgrInterop.OGR_G_AddPoint_2D(this.geometryHandle, x0, y0);
			}
		}
	}
}
