using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrPolygon : OgrSurface
	{
		#region constructors

		public OgrPolygon(IntPtr ptrPolygon)
			: base(ptrPolygon, OGRwkbGeometryType.wkbPolygon)
		{
		}

		public OgrPolygon()
			: base(OGRwkbGeometryType.wkbPolygon)
		{
		}

		#endregion

		#region public methods

		/// <summary>
		/// Adds linear ring object to the polygon object.
		/// </summary>
		/// <param name="linearRing"></param>
		public void AddRingDirectly(OgrLinearRing linearRing)
		{
			OGRErr error = OgrInterop.OGR_G_AddGeometryDirectly(this.geometryHandle,
				linearRing.geometryHandle);

			if (error != OGRErr.None)
			{
				string message = "Could not add Linear Ring to Polygon. " +
					error.ToString();

				throw new OgrException(message);
			}
		}

		/// <summary>
		/// Gets the polygon's exterior linear ring.
		/// </summary>
		/// <returns>OgrLinearRing object if one exists. Null otherwise.</returns>
		public OgrLinearRing GetExteriorRing()
		{
			IntPtr ptrLinearRing = OgrInterop.OGR_G_GetGeometryRef(this.geometryHandle, 0);
			if (ptrLinearRing != IntPtr.Zero)
			{
				return new OgrLinearRing(ptrLinearRing);
			}
			return null;
		}

		#endregion
	}
}
