using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrFeatureDefinition
	{
		internal HandleRef featureDefinitionHandle;

		#region constructors

		public OgrFeatureDefinition(IntPtr ptrFeatureDefinition)
		{
			this.featureDefinitionHandle = new HandleRef(this, ptrFeatureDefinition);
		}

		public OgrFeatureDefinition(string featureDefinitionName)
		{
			IntPtr ptrFD = OgrInterop.OGR_FD_Create(featureDefinitionName);
			if (ptrFD != IntPtr.Zero)
			{
				this.featureDefinitionHandle = new HandleRef(this, ptrFD);
			}
			else
			{
				throw new OgrException("Could not create feature definition");
			}
		}

		#endregion

		#region public properties

		/// <summary>
		/// Gets the number of fields in the field definition.
		/// </summary>
		public int NumberFields
		{
			get
			{
				return OgrInterop.OGR_FD_GetFieldCount(this.featureDefinitionHandle);
			}
		}

		#endregion

		#region public methods

		/// <summary>
		/// Gets the field definition.
		/// </summary>
		/// <param name="fieldIndex">The field index.</param>
		/// <returns>Field definition object if successful. Null otherwise.</returns>
		public OgrFieldDefinition GetFieldDefinition(int fieldIndex)
		{
			IntPtr ptrFD = OgrInterop.OGR_FD_GetFieldDefn(this.featureDefinitionHandle, fieldIndex);
			if (ptrFD != IntPtr.Zero)
			{
				return new OgrFieldDefinition(ptrFD);
			}
			return null;
		}

		#endregion
	}
}
