using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public enum OGRErr
	{
		None = 0,
		NotEnoughDataToDeserialize,
		NotEnoughMemory,
		UnsupportedGeometryType,
		UnsupportedOperation,
		CorruptData,
		Failure,
		UnsupportedSRS
	};

	internal class OgrInterop
	{
		static OgrInterop()
		{
		}

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGRRegisterAll();

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGRGetDriver(int iDriver);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OGR_Dr_CreateDataSource(HandleRef hDriver,
			string pszName, ref string[] papszOptions);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGR_Dr_DeleteDataSource(HandleRef hDriver);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OGR_DS_CreateLayer(HandleRef hDS,
			string pszName, HandleRef hSpatialRef, OGRwkbGeometryType eType,
			ref string[] papszOptions);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGR_L_CreateField(HandleRef hLayer,
			HandleRef hField, int bApproxOK);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OGR_Fld_Create(string pszName, OGRFieldType eType);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_Fld_SetWidth(HandleRef hDefn, int nNewWidth);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_Fld_SetPrecision(HandleRef hDefn, int nPrecision);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_Fld_GetWidth(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_Fld_GetPrecision(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_F_Create(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_L_GetLayerDefn(HandleRef hLayer);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_G_CreateGeometry(OGRwkbGeometryType eType);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_G_AddPoint_2D(HandleRef hGeom, double dFX, double dFY);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGR_F_SetGeometryDirectly(HandleRef hFeat,
			HandleRef hGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGR_G_AddGeometryDirectly(HandleRef hGeom,
			HandleRef hNewSubGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern void OGR_F_SetStyleString(HandleRef hFeat,
			string pszStyle);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OGR_FD_Create(string pszName);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGR_L_CreateFeature(HandleRef hLayer, HandleRef hFeature);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_F_SetFieldDouble(HandleRef hFeat,
			int iField, double dfValue);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_F_SetFieldInteger(HandleRef hFeat,
			int iField, int nValue);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern void OGR_F_SetFieldString(HandleRef hFeat,
			int iField, string pszValue);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OGROpen(string pszName, int bUpdate,
			HandleRef pahDriverList);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_DS_GetLayerCount(HandleRef hDS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_DS_GetLayer(HandleRef hDS, int iLayer);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_L_ResetReading(HandleRef hLayer);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_L_GetNextFeature(HandleRef hLayer);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_FD_GetFieldCount(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_F_GetFieldAsInteger(HandleRef hFeat, int iField);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern double OGR_F_GetFieldAsDouble(HandleRef hFeat, int iField);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern string OGR_F_GetFieldAsString(HandleRef hFeat, int iField);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_F_GetGeometryRef(HandleRef hFeat);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern uint OGR_G_GetGeometryType(HandleRef hGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGR_G_GetPointCount(HandleRef hGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_G_GetPoint(HandleRef hGeom, int i,
			out double pdfX, out double pdfY, out double pdfZ);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OGR_F_Destroy(HandleRef hFeat);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_FD_GetFieldDefn(HandleRef hDefn, int iField);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OGRGetDriverCount();

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern string OGR_Dr_GetName(HandleRef hDriver);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRFieldType OGR_Fld_GetType(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern IntPtr OSRNewSpatialReference(string pszWKT);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern string OGR_DS_GetName(HandleRef hDS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OGRReleaseDataSource(HandleRef hDS);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern OGRErr OSRSetProjCS(HandleRef hSpatRef, string pszCS);

		[DllImport(Gdal.GdalInterOp.GdalDllName, CharSet = CharSet.Ansi)]
		internal static extern OGRErr OSRSetWellKnownGeogCS(HandleRef hSRS, string pszName);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OSRSetUTM(HandleRef hSRS, int nZone, int bNorth);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern OGRErr OSRValidate(HandleRef hSRS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OSRGetUTMZone(HandleRef hSRS, out int pbNorth);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_Fld_GetNameRef(HandleRef hDefn);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_G_GetGeometryRef(HandleRef hGeom, int iSubGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OGR_G_Clone(HandleRef hGeom);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern Int32 OCTTransform(HandleRef hCT, int nCount, [In, Out] double[] x,
			[In, Out] double[] y, [In, Out] double[] z);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OCTNewCoordinateTransformation(HandleRef hSourceSRS,
			HandleRef hTargetSRS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern IntPtr OSRClone(HandleRef hSRS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern int OSRDereference(HandleRef hSRS);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal static extern void OCTDestroyCoordinateTransformation(HandleRef hCT);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal unsafe static extern void OSRExportToWkt(HandleRef hSRS, char** pszSRS_WKT);

		[DllImport(Gdal.GdalInterOp.GdalDllName)]
		internal unsafe static extern void VSIFree(char* pData);
	}
}
