using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace GDALWrapper.OGR
{
	public class OgrSpatialReference
	{
		internal HandleRef spatialReferenceHandle;

		public bool IsValid
		{
			get
			{
				OGRErr error = OgrInterop.OSRValidate(this.spatialReferenceHandle);
				return (error == OGRErr.None);
			}
		}

		#region constructors

		public OgrSpatialReference()
		{
			IntPtr ptrSR = OgrInterop.OSRNewSpatialReference(String.Empty);
			if (ptrSR == IntPtr.Zero)
			{
				throw new Exception("Spatial reference could not be created.");
			}
			this.spatialReferenceHandle = new HandleRef(this, ptrSR);
		}

		public OgrSpatialReference(IntPtr ptrSRS)
		{
			this.spatialReferenceHandle = new HandleRef(this, ptrSRS);
		}

		// couldn't get this working
		// tried marshalling to ANSI ptr, string

		//public OgrSpatialReference(string wellKnownTextDefinition)
		//{
		//    IntPtr ptrSR = OgrInterop.OSRNewSpatialReference(wellKnownTextDefinition);
		//    if (ptrSR == IntPtr.Zero)
		//    {
		//        throw new Exception("Spatial reference could not be created.");
		//    }
		//    this.spatialReferenceHandle = new HandleRef(this, ptrSR);
		//}

		#endregion

		#region public methods

		public void SetProjectionCoordinateSystem(string coordSysName)
		{
			OGRErr error = OgrInterop.OSRSetProjCS(this.spatialReferenceHandle, coordSysName);

			if (error != OGRErr.None)
			{
				throw new OgrException(error);
			}
		}

		public void SetWellKnownGeographicCoordinateSystem(string coordSysName)
		{
			OGRErr error = OgrInterop.OSRSetWellKnownGeogCS(this.spatialReferenceHandle, coordSysName);
			if (error != OGRErr.None)
			{
				throw new OgrException(error);
			}
		}

		public void SetUTM(int zone, bool north)
		{
			OGRErr error = OgrInterop.OSRSetUTM(this.spatialReferenceHandle, zone, north ? 1 : 0);
			if (error != OGRErr.None)
			{
				throw new OgrException(error);
			}
		}

		public void GetUTMZone(out int zone, out bool north)
		{
			int northInt;
			zone = OgrInterop.OSRGetUTMZone(this.spatialReferenceHandle, out northInt);
			north = (northInt > 0);
		}

		public void Dereference()
		{
			OgrInterop.OSRDereference(this.spatialReferenceHandle);
		}

		public OgrSpatialReference Clone()
		{
			return new OgrSpatialReference(OgrInterop.OSRClone(this.spatialReferenceHandle));

		}

		public unsafe string GetWktName()
		{
			//IntPtr wktNamePtr = IntPtr.Zero;
			char* wktNamePtr = null;
			OgrInterop.OSRExportToWkt(this.spatialReferenceHandle, &wktNamePtr);
			IntPtr wktIntPtr = new IntPtr(wktNamePtr);
			string wktName = Marshal.PtrToStringAnsi(wktIntPtr);
			OgrInterop.VSIFree(wktNamePtr);
			return wktName;
		}

		#endregion
	}
}
